package RPM::Source::Tools::BuildReqStorage;

use strict;
use warnings;

use Carp;
use Clone;
our @ISA=qw/Clone/;

our $VERSION=0.101;
# there is hardcoded rpm limit, about 5000. but more than 800 is not practical.
my $RPM_BR_STRING_LIMIT=800;

sub new {
    my $class = shift;
    my $def_ignore_buildreq = 
	(defined $DistroMap::default_destrepo and $DistroMap::default_destrepo eq 'altlinux') ? 
	# sendmail: /usr/sbin/sendmail is in sendmail-submit for ALT, in sendmail for generic...
	# /usr/bin/python python-base; banned by python3 migration policy
	# perl-base: in base build environment now
        {map {$_=>1} qw!glibc-devel coreutils sh make /usr/sbin/sendmail /usr/bin/python python-base perl-base!}
        : {};
    my $self = {
	-entry_comment=> 'SourceDeps', # TODO: timestamp?
	-entry_type=>'oneline',
	-post_header_comment => undef,
	-pre_footer_comment => undef,
	-with=>{},
	-buildreq=>{},
	-buildprereq=>{},
	-ignore_buildreq=>$def_ignore_buildreq,
	-requires=>{},
	-provides=>{},
#	-is_arch_dependent => undef,
	@_
    };
    bless $self, $class;

    return $self;
}

# used in RPM::Source::Dependency::Analyzer::Resolver
sub new_from_br_brpre {
    my $class = shift;
    my $br = shift;
    my $br_pre = shift;
    return $class->new(
	-buildreq=>$br,
	-buildprereq=>$br_pre,
	@_,
	);
}

# sub arch_dependent {
#     my ($self, $val)=@_;
#     $self->{-is_arch_dependent}=$val if defined $val;
#     return $self->{-is_arch_dependent};
# }

sub __init_with {
    my ($self,$with)=@_;
    my $withref=$self->{-with}->{$with};
    foreach my $key (qw/-buildreq -buildprereq -requires -provides/) {
	$withref->{$key}||={};
    }
}

sub add_depreq {
    my $self=shift;
    my $opt={
	-for => '',
	@_
    };
    my $txtfor=$opt->{-for};
    my $domain=$self;
    my $optwith=$opt->{-with};
    if ($optwith) {
	$self->__init_with($optwith);
	$domain=$self->{-with}->{$optwith};
    }
    foreach my $key (qw/-buildreq -buildprereq -requires -provides/) {
	my $list=$opt->{$key};
	my $deps=$domain->{$key};
	foreach my $dep (@$list) {
	    $deps->{$dep}=$txtfor;
	}
    }
}

sub ignore_buildreq {
    my ($self, @names)=@_;
    foreach my $name (@names) {
	$self->{-ignore_buildreq}->{$name}=1;
    }
}

sub add_buildreq {
    my ($self, @names)=@_;
    my $deps=$self->{-buildreq};
    foreach my $name (@names) {
	$deps->{$name}||='';
    }
}

sub add_buildreq_for {
    my ($self, $reason, @names)=@_;
    my $deps=$self->{-buildreq};
    &__add_reason_names($deps,$reason, @names);
}

sub __add_reason_names {
    my ($deps, $reason, @names)=@_;
    foreach my $name (@names) {
	my $oldreason=$deps->{$name};
	if ($oldreason) {
	    $oldreason.='; '.$reason;
	} else {
	    $oldreason=$reason;
	}
	$deps->{$name}=$oldreason;
    }
}

sub add_pre_buildreq {
    my ($self, @names)=@_;
    my $deps=$self->{-buildreq};
    my $predeps=$self->{-buildprereq};
    foreach my $name (@names) {
	$deps->{$name}||='';
	$predeps->{$name}||='';
    }
}

sub add_pre_buildreq_for {
    my ($self, $reason, @names)=@_;
    my $deps=$self->{-buildreq};
    my $predeps=$self->{-buildprereq};
    foreach my $name (@names) {
	$deps->{$name}||='';
	my $oldreason=$predeps->{$name};
	if ($oldreason) {
	    $oldreason.='; '.$reason;
	} else {
	    $oldreason=$reason;
	}
	$predeps->{$name}=$oldreason;
    }
}

sub has_buildreq {
    my ($self,$name)=@_;
    my $deps=$self->{-buildreq};
    return defined $deps->{$name};
}

sub has_pre_buildreq {
    my ($self,$name)=@_;
    my $predeps=$self->{-buildprereq};
    return defined $predeps->{$name};
}

sub has_ignore_buildreq {
    my ($self,$name)=@_;
    my $deps=$self->{-ignore_buildreq};
    return defined $deps->{$name};
}

sub get_list_buildreq {
    my ($self)=@_;
    return keys(%{$self->{-buildreq}});
}

sub get_list_pre_buildreq  {
    my ($self)=@_;
    return keys(%{$self->{-buildprereq}});
}

sub get_list_ignore_buildreq  {
    my ($self)=@_;
    return keys(%{$self->{-ignore_buildreq}});
}

sub del_buildreq {
    my ($self,$name)=@_;
    my $deps=$self->{-buildreq};
    delete $deps->{$name};
}

sub del_pre_buildreq {
    my ($self,$name)=@_;
    my $predeps=$self->{-buildprereq};
    delete $predeps->{$name};
}

sub del_ignore_buildreq {
    my ($self,$name)=@_;
    my $deps=$self->{-ignore_buildreq};
    delete $deps->{$name};
}

sub add_requires {
    my ($self, @names)=@_;
    my $deps=$self->{-requires};
    foreach my $name (@names) {
	$deps->{$name}||='';
    }
}

sub add_requires_for {
    my ($self, $reason, @names)=@_;
    my $deps=$self->{-requires};
    &__add_reason_names($deps,$reason, @names);
}

sub del_requires {
    my ($self,$name)=@_;
    my $deps=$self->{-requires};
    delete $deps->{$name};
}

sub add_provides {
    my ($self, @names)=@_;
    my $deps=$self->{-provides};
    foreach my $name (@names) {
	$deps->{$name}||='';
    }
}

sub add_provides_for {
    my ($self, $reason, @names)=@_;
    my $deps=$self->{-provides};
    &__add_reason_names($deps,$reason, @names);
}

sub del_provides {
    my ($self,$name)=@_;
    my $deps=$self->{-provides};
    delete $deps->{$name};
}


# used in buildreq_src only: starting point to refactor;
sub get_entry {
    my ($self,$spec_current_state)=@_;
    my $existing=$spec_current_state->{-manual_buildreq};
    my $existing_pre=$spec_current_state->{-manual_buildreq_pre};
    my $output='';
    my $ignore=$self->{-ignore_buildreq};
    my $breq=$self->{-buildreq};
    my $prebreq=$self->{-buildprereq};
    if ($self->{-entry_type} eq 'oneline') {
	$output.=&__oneline_brpair_output($prebreq,$breq,$ignore,$existing_pre,$existing);
	$output.=&__oneline_output('Provides: ',$self->{-provides});
	$output.=&__oneline_output('Requires: ',$self->{-requires});
	my $selfwith=$self->{-with};
	foreach my $with (keys(%$selfwith)) {
	    my $domain=$selfwith->{$with};
	    my $optout=&__oneline_brpair_output($domain->{-buildprereq},$domain->{-buildreq},$ignore,$existing_pre,$existing);
	    $optout.=&__oneline_output('Provides: ',$domain->{-provides});
	    $optout.=&__oneline_output('Requires: ',$domain->{-requires});
	    $output.='%if_with '.$with."\n".$optout.'%endif'."\n" if $optout;
	}
    } elsif ($self->{-entry_type} eq 'multiline') {
	$output.=&__multiline_brpair_output($prebreq,$breq,$ignore,$existing_pre,$existing);
	$output.=&__multiline_output('Provides: ',$self->{-provides});
	$output.=&__multiline_output('Requires: ',$self->{-requires});
	my $selfwith=$self->{-with};
	foreach my $with (keys(%$selfwith)) {
	    my $domain=$selfwith->{$with};
	    my $optout=&__multiline_brpair_output($domain->{-buildprereq},$domain->{-buildreq},$ignore,$existing_pre,$existing);
	    $optout.=&__multiline_output('Provides: ',$domain->{-provides});
	    $optout.=&__multiline_output('Requires: ',$domain->{-requires});
	    $output.='%if_with '.$with."\n".$optout.'%endif'."\n" if $optout;
	}
    } else {
	die "unknown/unsupported entry type ".$self->{-entry_type};
    }
    if ($output) {
	my $post_header_comment=$self->{-post_header_comment};
	$output=$post_header_comment.$output if $post_header_comment;
	my $pre_footer_comment=$self->{-pre_footer_comment};
	$output.=$pre_footer_comment if $pre_footer_comment;
    }
    $output=$self->__entry_header()."\n".$output.$self->__entry_footer()."\n" if $output;
    return $output;
}

#------------------------------------------------------------------
# private methods
#------------------------------------------------------------------
sub __multiline_brpair_output {
    my ($prebreq,$breq,$ignore,$existing_pre,$existing)=@_;
    my ($predeps,$deps)=&__filter_brpair_output($prebreq,$breq,$ignore,$existing_pre,$existing);
    return &__multiline_output('BuildRequires(pre): ',$predeps).
	&__multiline_output('BuildRequires: ',$deps).
	'';
}

sub __oneline_brpair_output {
    my ($prebreq,$breq,$ignore,$existing_pre,$existing)=@_;
    my ($predeps,$deps)=&__filter_brpair_output($prebreq,$breq,$ignore,$existing_pre,$existing);
    return &__oneline_output('BuildRequires(pre): ',$predeps).
	&__oneline_output('BuildRequires: ',$deps).
	'';
}

sub __oneline_output {
    my ($prefix,$deps)=@_;
    my $output='';
    map {$output.= $prefix.$_."\n"} &__limited_join($RPM_BR_STRING_LIMIT,' ',sort {$a cmp $b} keys(%$deps));
    return $output;
}

sub __multiline_output {
    my ($prefix,$deps)=@_;
    my $output='';
    foreach my $dep (sort {$a cmp $b} keys(%$deps)) {
	next unless $dep;
	my $reason=$deps->{$dep};
	$output.= '# '.$reason."\n" if $reason;
	$output.= $prefix.$dep."\n";
    }
    return $output;
}

sub __filter_brpair_output {
    my ($prebreq,$breq,$ignore,$existing_pre,$existing)=@_;
    my (%predeps,%deps);
    # beware of '' reasons!
    foreach my $predep (keys(%$prebreq)) {
	$predeps{$predep}=$prebreq->{$predep} if not defined $ignore->{$predep} and not defined $existing_pre->{$predep};
    }
    foreach my $dep (keys(%$breq)) {
	$deps{$dep}=$breq->{$dep} if not defined $ignore->{$dep} and not defined $prebreq->{$dep} and not defined $existing->{$dep};
    }
    return \%predeps, \%deps;
}
#------------------------------------------------------------------
# end private methods
#------------------------------------------------------------------

# used nowhere outside this module
sub get_spec_entry {
    my $self=shift;
    my $spec=shift;
    croak "Oops! get_spec_entry: odd number in opts hash assignment" if scalar(@_)%2!=0;
    my %opt=(@_);
    my $spec_current_state=$self->collect_spec_current_deps_and_sourcedep_entry($spec);
    if ($opt{-merge_sourcedep_entry}) {
	# merge $old,$old_pre with $self
	&__merge_keyhash($self->{-buildreq},$spec_current_state->{-sourcedep_entry_buildreq});
	&__merge_keyhash($self->{-buildprereq},$spec_current_state->{-sourcedep_entry_buildreq_pre});
	&__merge_keyhash($self->{-requires},$spec_current_state->{-sourcedep_entry_requires});
	&__merge_keyhash($self->{-provides},$spec_current_state->{-sourcedep_entry_provides});
    }
    return $self->get_entry($spec_current_state);
}

sub update_spec_entry {
    my $self=shift;
    my $spec=shift;
    croak "Oops! get_spec_entry: odd number in opts hash assignment" if scalar(@_)%2!=0;
    #my %opt=(@_);
    my $spec_entry=$self->get_spec_entry($spec,@_);

    my $mainsec=$spec->main_section;
    my @main_body_copy=@{$mainsec->get_bodyref};
    my $update_failed;
    my $is_old_entry_found=0;
    my $begin_anchor=$self->__entry_header_anchor;
    my $end_anchor=$self->__entry_footer_anchor;

    $mainsec->map_body (sub {
	# multiple entries can appear
	#return if $is_old_entry_found>1;
	if (/$begin_anchor/) {
	    $is_old_entry_found++; $_=''; return;
	} elsif (0==$is_old_entry_found % 2) {
	    return;
	}
	if (/^BuildRequires/) {
	    #print "BR:$_";
	    $_=''; return;
	} elsif (/$end_anchor/) {
	    # insert new entry at first found old entry
   	    $_=$spec_entry if 1==$is_old_entry_found;
	    $is_old_entry_found++;
	    return;
	} elsif (/^#/) { # other comment
	    $_=''; return;
	} elsif (/^\%(?:if|endif)/) { # RPM conditoinals are allowed too
	    $_=''; return;
	} else {
	    $update_failed=1;
	}
			});

    if ($is_old_entry_found % 2 == 1 or $update_failed) {
	warn 'BuildReqStorage: old spec entry '.$self->__entry_header()." is broken, update failed\n";
	$mainsec->set_body(\@main_body_copy);
	$is_old_entry_found=0;
    }
    $mainsec->unshift_body($spec_entry) if $is_old_entry_found == 0;
}

#------------------------------------------------------------------
# protected; used in buildreq-src and DistroMap::SourceAnalyzer
#------------------------------------------------------------------
# out/outpre are collected BR:/BR(pre).
# old/oldpre are  BR:/BR(pre) that belong to old existing entry
sub collect_spec_current_deps_and_sourcedep_entry {
    my ($self,$spec)=@_;
    # TODO: possible option
    my $is_merge_old_entry=0;
    my ($out,$outpre)=({},{});
    my ($requires,$provides)=({},{});

    my $is_old_entry_found=0;
    my $begin_anchor=$self->__entry_header_anchor;
    my $end_anchor=$self->__entry_footer_anchor;
    my ($old,$old_pre)=({},{});
    my ($old_requires,$old_provides)=({},{});
    if ($spec) {
	foreach my $section ($spec->get_sections()) {
	    next if $section->get_type() ne 'package';
	    if (not $section->is_main_section) {
		foreach (@{$section->get_bodyref()}) {
		    &__add_dep($_,$out,$outpre,$requires,$provides);
		}
	    } else {
		$section->map_body (sub {
		    # support for merging multiple entries
		    #if ($is_old_entry_found > 1) {
		    #	&__add_dep($_,$out,$outpre); } els
		    if (/$begin_anchor/) {
			$is_old_entry_found++;
		    } elsif (0 == $is_old_entry_found % 2) {
			&__add_dep($_,$out,$outpre,$requires,$provides);
		    } elsif (/$end_anchor/) {
			$is_old_entry_found++;
		    } else { # (/^BuildRequires/) { 
			# we are inside an entry; Let us support even a broken entry.
			#print "ENTRY:$_";
			&__add_dep($_,$old,$old_pre,$old_requires,$old_provides);
		    }
				    });
		
	    }
	}
	if ($is_merge_old_entry) {
	    map {$outpre->{$_}=1 } keys (%$old_pre);
	    map {$out->{$_}=1 } keys (%$old);
	    map {$requires->{$_}=1 } keys (%$old_requires);
	    map {$provides->{$_}=1 } keys (%$old_provides);
	}
    }
    
    return {
	-manual_buildreq => $out,
	-manual_buildreq_pre => $outpre,
	-manual_requires => $requires,
	-manual_provides => $provides,
	-has_sourcedep_entry => $is_old_entry_found,
	-sourcedep_entry_buildreq => $old,
	-sourcedep_entry_buildreq_pre => $old_pre,
	-sourcedep_entry_requires => $old_requires,
	-sourcedep_entry_provides => $old_provides,
    };
}


#------------------------------------------------------------------
# private methods
#------------------------------------------------------------------

sub __entry_header_anchor {
    my ($self)=@_;
    return '^# BEGIN '."\Q$self->{-entry_comment}\E".'\('.$self->{-entry_type}.'\):';
}

sub __entry_footer_anchor {
    my ($self)=@_;
    return '^# END';
}

sub __entry_header {
    my ($self)=@_;
    return '# BEGIN '.$self->{-entry_comment}.'('.$self->{-entry_type}.'):';
}

sub __entry_footer {
    my ($self)=@_;
    return '# END '.$self->{-entry_comment}.'('.$self->{-entry_type}.')';
}

#------------------------------------------------------------------
# private static
#------------------------------------------------------------------

sub __merge_keyhash {
    my ($base, $merged)=@_;
    foreach my $key (keys(%$merged)) {
	$base->{$key}=$merged->{$key};
    }
}

sub __limited_join {
    my ($limit,$sep,@array)=@_;
    my @output;
    return @output unless @array;
    my $joined=shift @array;
    while (@array) {
	if (length($joined) > $limit) {
	    push @output, $joined;
	    $joined=shift @array;
	} else {
	    $joined.=$sep.shift(@array);
	}
    }
    push @output, $joined;
    return @output;
}

sub __add_dep {
    my ($line,$out,$outpre,$req,$prov)=@_;
    if ($line=~/^\s*(\%\{\?!?[^:]+:)?\s*((?:Requires|Provides|BuildReq|BuildRequires|BuildPreReq|BuildPrereqs)\s*(?:\(\w+\))?)\s*:(\s*)(.*\S)\s*$/i) {
	my $macro=$1;
	my $Tag=$2;
	my $space=$3;
	my $reqlist=$4;
	my $endmacro='';
	if ($macro) {
	    $reqlist=~s/\s*(\})\s*$//;
	    $endmacro=$1;
	}
	my $names = &__get_pkg_names($reqlist);
	if ($Tag=~/^Requires/i) {
	    map {$req->{$_}=1} @$names;
	} elsif ($Tag=~/^Provides/i) {
	    map {$prov->{$_}=1} @$names;
	} elsif ($Tag=~/pre/i) {
	    map {$out->{$_}=1} @$names;
	    map {$outpre->{$_}=1} @$names;
	} else {
	    map {$out->{$_}=1} @$names;
	}
    }
}

sub __get_pkg_names {
    my ($line)=@_;
    # hack around rpm4/add_changelog error
    #error: Dependency tokens must not contain '%<=>' symbols: Requires: icu4j-eclipse > = 3.6.1
    $line =~ s/\>\s+=/>=/g;
    $line =~ s/\<\s+=/<=/g;
    # TODO: altlinux specific
    #$line =~ s/\%\{dist\}//g;
    my @in = split /[\s,]+/, $line;
    my @out;
    while (@in) {
	my $token = shift @in;
	if ($token =~/^[<=>]+$/) {
	    shift @in;
	} else {
	    push @out, $token;
	}
    }
    return \@out;
};

1;
