package RPM::Source::Transformation::CheckIfChangedPlayer;

use strict;
use warnings;
use Carp;

use RPM::Source::Transformation::Component;
use RPM::Source::Transformation::Factory;
our @ISA=qw/RPM::Source::Transformation::Component/;

push @RPM::Source::TransformContainer::SUPPORT_CLASSES, __PACKAGE__;
$RPM::Source::TransformContainer::PLAYER=__PACKAGE__;

our $verbose=0;
our $opt_use_check_if_changed;

sub process_options {
    # override default player if loaded together
    $RPM::Source::TransformContainer::PLAYER=__PACKAGE__ if $opt_use_check_if_changed;
}

sub apply_collection {
    my (undef,$spec,$parent,$opt,$resourceobj,$actions)=@_;

    my $before=$spec->get_spec();
    my @transformations=$actions->get;
    my $raise_release=pop @transformations;
    die "Oops! internal: not a raise release ",$raise_release->{-name} if $raise_release->{-name} ne 'raise release';

    foreach my $transformation (@transformations) {
	my $exit_code=$transformation->apply($spec,$parent,$opt,$resourceobj);
	if ($exit_code == TRANSFORMATION_FAILURE or $exit_code == TRANSFORMATION_SKIP) {
	    return $exit_code;
	} elsif (not defined $exit_code or $exit_code != TRANSFORMATION_OK) {
	    warn 'transformation ',$transformation->{-name}," returned invalid exit code\n";
	}
    }

    if ($before ne $spec->get_spec()) {
	$raise_release->apply($spec,$parent,$opt,$resourceobj);
	return TRANSFORMATION_OK;
    }
    print STDERR "CheckIfChangedPlayer: skipped ", $spec->macros->macro_subst('%name')," - nothing changed\n";
    return TRANSFORMATION_SKIP;
}

1;
