package RPM::Source::Transformation::DefaultPlayer;

use strict;
use warnings;
use Carp;

use RPM::Source::Transformation::Component;
use RPM::Source::Transformation::Factory;
our @ISA=qw/RPM::Source::Transformation::Component/;

push @RPM::Source::TransformContainer::SUPPORT_CLASSES, __PACKAGE__;
$RPM::Source::TransformContainer::PLAYER=__PACKAGE__;

our $VERSION=0.001;
our $verbose=0;
my @opt_player_skip_transformation;
my %player_skip_transformation;

sub print_longopt {
    print "
  Transformation Player options:
	--player-skip-transformation <name1,...>	Skip named transformation(s)
";
}

our @LONGOPT=(
    'player-skip-transformation=s'   => \@opt_player_skip_transformation,
    );

sub process_options {
    map {$player_skip_transformation{$_}=1} map {split(',',$_)} @opt_player_skip_transformation;
}

sub apply_collection {
    my (undef,$spec,$parent,$opt,$resourceobj,$actions)=@_;
    print STDERR "DefaultPlayer: entering apply_collection.\n" if $verbose;
    foreach my $transformation ($actions->get) {
	my $name=$transformation->{-name};
	if ($player_skip_transformation{$name}) {
	    print STDERR 'DefaultPlayer: skipped ',$name,"\n" if $verbose;
	    next;
	} else {
	    print STDERR 'DefaultPlayer: applying ',$name,"...\n" if $verbose;
	}
	my $exit_code=$transformation->apply($spec,$parent,$opt,$resourceobj);
	if ($exit_code == TRANSFORMATION_FAILURE) {
	    print STDERR 'DefaultPlayer: ',$transformation->{-name},": returned TRANSFORMATION_FAILURE\n" if $verbose;
	    return TRANSFORMATION_FAILURE;
	} elsif ($exit_code == TRANSFORMATION_SKIP) {
	    print STDERR 'DefaultPlayer: ',$transformation->{-name},": returned TRANSFORMATION_SKIP\n" if $verbose;
	    return TRANSFORMATION_SKIP;
	} elsif (not defined $exit_code or $exit_code != TRANSFORMATION_OK) {
	    warn 'transformation ',$transformation->{-name}," returned invalid exit code\n";
	}
    }
    print STDERR "DefaultPlayer: leaving apply_collection.\n" if $verbose;
    return TRANSFORMATION_OK;
}

1;
