package RPM::Source::Transformation::Factory;

use strict;
use warnings;

use RPM::Source::Transformation::Component;
use RPM::Source::Transformation::OptHandlerWrapper;

our $VERSION=0.017;
our @ISA=qw(RPM::Source::Transformation::Component);

use Exporter 'import';
our @EXPORT = qw(TRANSFORMATION_FAILURE TRANSFORMATION_OK TRANSFORMATION_SKIP
PRIORITY_AT_BEGIN
PRIORITY_PRE_BUNDLE
PRIORITY_BUNDLE_SOURCE
PRIORITY_BUNDLE_POST_SOURCE
PRIORITY_BUNDLE_PRECOLLECT
PRIORITY_BUNDLE_COLLECT
PRIORITY_BUNDLE_POSTCOLLECT
PRIORITY_BUNDLE_VERIFY_METADATA
PRIORITY_BUNDLE_SETUP
PRIORITY_BUNDLE_RESET
PRIORITY_BUNDLE_PREFILL
PRIORITY_BUNDLE_FILL
PRIORITY_BUNDLE_POSTFILL
PRIORITY_POST_BUNDLE
PRIORITY_SRPM_REINIT
PRIORITY_SRPM_REINIT_FALLBACK
PRIORITY_HOOK_AUTOLOAD
PRIORITY_SRPM_REINIT_HOOKS
PRIORITY_SRPM_POST_REINIT
PRIORITY_SRPM_RESET
PRIORITY_SRPM_AT_BEGIN
PRIORITY_SRPM_POST_BEGIN
PRIORITY_PREHOOKS
PRIORITY_PRECONVERT
PRIORITY_CONVERT
PRIORITY_FILTER_PREDISTROMAP
PRIORITY_FILTER_DISTROMAP
PRIORITY_FILTER_POSTDISTROMAP
PRIORITY_POSTCONVERT
PRIORITY_HOOKS
PRIORITY_POSTHOOKS
PRIORITY_COMMANDLINE
PRIORITY_DEFAULT
PRIORITY_POSTPROCESS
PRIORITY_PRERELEASE
PRIORITY_RAISE_RELEASE
PRIORITY_TRY_RPMBUILD_PREP_BEFORE
PRIORITY_TRY_RPMBUILD_PREP_EXEC
PRIORITY_TRY_RPMBUILD_PREP_AFTER
PRIORITY_TRY_RPMBUILD_PREP_PROCESS
PRIORITY_TRY_RPMBUILD_PREP_POSTPROCESS
PRIORITY_TRY_RPMBUILD_PREP_CLEANUP_BEFORE
PRIORITY_TRY_RPMBUILD_PREP_CLEANUP
PRIORITY_TRY_RPMBUILD_PREP_CLEANUP_AFTER
PRIORITY_AT_END
);

use constant {
	TRANSFORMATION_FAILURE	=> RPM::Source::TransformContainer::TRANSFORMATION_FAILURE,
	TRANSFORMATION_SKIP	=> RPM::Source::TransformContainer::TRANSFORMATION_SKIP,
	TRANSFORMATION_OK	=> RPM::Source::TransformContainer::TRANSFORMATION_OK,

	PRIORITY_AT_BEGIN	=> 900,
	# reserved for future use
	#PRIORITY_POST_BEGIN	=> 890,
	# bundle import...
	PRIORITY_PRE_BUNDLE	=> 520, # before main
	PRIORITY_BUNDLE_SOURCE	=> 510, # process input and setup SOURCES file(s)
	PRIORITY_BUNDLE_POST_SOURCE	=> 500, # submit processed source to input bundle resource
	PRIORITY_BUNDLE_PRECOLLECT	=> 490, # collecting parent quirks used (pkg_macros ...)
	PRIORITY_BUNDLE_COLLECT		=> 480, # collect in main, like in PerlBuilder.pm
	PRIORITY_BUNDLE_POSTCOLLECT	=> 470, # after main, still collecting metadata
	PRIORITY_BUNDLE_VERIFY_METADATA	=> 460, # after collecting metadata
	PRIORITY_BUNDLE_SETUP	=> 450, # apply metadata to spec
	PRIORITY_BUNDLE_RESET	=> 440, # reset; sync our macros with spec
	PRIORITY_BUNDLE_PREFILL	=> 430, # clean up parent quirks if any ...
	PRIORITY_BUNDLE_FILL	=> 420, # fill %prep,%build,%install,%files in main
	PRIORITY_BUNDLE_POSTFILL=> 410, # fill %prep,%build,%install,%files in plugins
	PRIORITY_POST_BUNDLE	=> 400,
	# reinit stage (optional):
	# Name can be missing, as in %fontpkg, %forgemeta, etc
	# so we need to re-initialize %name, Name and Datapath resource
	PRIORITY_SRPM_REINIT		=> 270,
	PRIORITY_SRPM_REINIT_FALLBACK	=> 260,
	PRIORITY_HOOK_AUTOLOAD		=> 250,
	PRIORITY_SRPM_REINIT_HOOKS	=> 240,
	# at this step, Name and Datapath resource are set, creating and fixing subpackages ...
	PRIORITY_SRPM_POST_REINIT	=> 230,
	# reset spec sections and macros
	PRIORITY_SRPM_RESET		=> 220,
	# end reinit stage (optional):
	# srpmtool stage
	PRIORITY_SRPM_AT_BEGIN		=> 210,
	PRIORITY_SRPM_POST_BEGIN	=> 200,
	PRIORITY_PREHOOKS		=> 100,
	PRIORITY_PRECONVERT		=>  90,
	PRIORITY_CONVERT		=>  80,
	PRIORITY_FILTER_PREDISTROMAP	=>  70,
	PRIORITY_FILTER_DISTROMAP	=>  60,
	PRIORITY_FILTER_POSTDISTROMAP	=>  50,
	PRIORITY_POSTCONVERT	=>  40,
	PRIORITY_HOOKS		=>  30,
	PRIORITY_POSTHOOKS	=>  20,
	PRIORITY_COMMANDLINE	=>  10,
	PRIORITY_DEFAULT	=>  0,
	PRIORITY_POSTPROCESS	=> -10,
	PRIORITY_TRY_RPMBUILD_PREP_BEFORE	=>  -100,
	PRIORITY_TRY_RPMBUILD_PREP_EXEC	=>  -110,
	PRIORITY_TRY_RPMBUILD_PREP_AFTER	=>  -120,
	PRIORITY_TRY_RPMBUILD_PREP_PROCESS	=>  -130,
	PRIORITY_TRY_RPMBUILD_PREP_POSTPROCESS	=>  -140,
	PRIORITY_TRY_RPMBUILD_PREP_CLEANUP_BEFORE	=>  -150,
	PRIORITY_TRY_RPMBUILD_PREP_CLEANUP	=>  -160,
	PRIORITY_TRY_RPMBUILD_PREP_CLEANUP_AFTER	=>  -170,
	PRIORITY_PRERELEASE	=> -200,
	# warning! DiffWritePlayer expect raise_release to be last;
	# any not PRIORITY_AT_END transformation should be added to DiffWritePlayer list of exceptions.
	PRIORITY_RAISE_RELEASE	=> -210,
	# note: PRIORITY_AT_END should never change spec,
	# but work as destructor for some internal temporary state or resource.
	PRIORITY_AT_END		=> -220,
};

sub init_factory {}
sub transformations {
    return;
}

1;
