package RPM::Source::Transformation::Factory::Buildreq;

use strict;
use warnings;
use Carp;

use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::Resource::FlagLoader;
use RPM::Source::Transformation::Resource::Buildreq;

our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $VERSION=0.003;
# $verbose=0;
our $opt_buildreq_merge_sourcedep_entry;

my (@opt_ignore_buildreq,@opt_add_buildreq);

sub print_longopt {
    print "  Command Line options:
	--add-buildreq '<expr>' add <expr> to sourcedep BuildRequires entry.
	--ignore-buildreq '<expr>' remove <expr> from sourcedep BuildRequires entry.
	--merge-old-sourcedep merge old sourcedep BuildRequires entry. Default is no.
";
}

our @LONGOPT=(
    'merge-old-sourcedep!'		=> \$opt_buildreq_merge_sourcedep_entry,
    'add_buildreq|add-buildreq=s'	=> \@opt_add_buildreq,
    'ignore_buildreq|ignore-buildreq=s'	=> \@opt_ignore_buildreq,
);

&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('buildreq/ignore.txt',FLAG_RESOURCE_LIST_AS_ARRAY);
&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('buildreq/add.txt',FLAG_RESOURCE_LIST_AS_ARRAY);

sub register_messages {
    my ($self,$extopt)=@_;
    ### TODO: use some fixed keys hash here
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-opt_buildreq_merge_sourcedep_entry => $opt_buildreq_merge_sourcedep_entry,
    });
}

sub transformations {
    return RPM::Source::Transformation::OptHandlerWrapper->new(
	-priority => PRIORITY_PRERELEASE,
	-name => 'add_buildreq',
	-opt_handler_sub=>\&opt_handler);
}

sub opt_handler {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $merge=$opt->{-opt_buildreq_merge_sourcedep_entry};
    # or we will wipe old entry in tools
    if (@opt_add_buildreq or @opt_ignore_buildreq) {
	$merge=1 unless defined $merge;
	# this will create buildreq, setting __buildreq
	$resourceobj->buildreq->add_buildreq_for('command line',@opt_add_buildreq);
	$resourceobj->buildreq->ignore_buildreq(@opt_ignore_buildreq);
    }
    my $buildreq=$resourceobj->{'__buildreq'};
    if ($buildreq) {
	# ignore flags if there is no buildreq
	my $ignoreref=$resourceobj->get_flagdb('buildreq/ignore.txt');
	my $addref=$resourceobj->get_flagdb('buildreq/add.txt');
	$resourceobj->buildreq->add_buildreq_for('flags',@$addref) if $addref and @$addref;
	$resourceobj->buildreq->ignore_buildreq(@$ignoreref) if $ignoreref and @$ignoreref;
	$buildreq->update_spec_entry($spec, -merge_sourcedep_entry => $merge);
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;
