package RPM::Source::Transformation::Factory::CheckNVR;

use strict;
use warnings;

use Source::Shared::Resource::Verbose;
#use RPM::Source::Transformation::Resource::NVR;
use RPM::Source::Transformation::Component::InputNameCommon;
use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $opt_check_nvr=0;
# for srpmimport, reserved for future use
our $opt_use_parent=0;
    
sub print_longopt {
    print "  Name Input options:
    --check-nvr     check that actual name-version-release matches expected.
";
}

our @LONGOPT=(
	"check-nvr!"   => \$opt_check_nvr,
    );

sub transformations {
    return unless $opt_check_nvr;
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    # after PRIORITY_COMMANDLINE (--url cmdline option)
	    -priority => PRIORITY_DEFAULT,
	    -name => 'merge parent Url',
	    -opt_handler_sub=>\&opt_handler_merge_parent_url);
}

sub opt_handler_merge_parent_url {
    my (undef,$spec,$parent,$converter_opt,$resourceobj)=@_;
    my ($expected_name,$expected_version,$expected_release);

    $expected_name=$converter_opt->{-input_handler_arg_input_name};
    return RPM::Source::TransformContainer::TRANSFORMATION_OK unless $expected_name;
    my $expected_nvr_string=$converter_opt->{-input_handler_arg_input_nvr};
    if ($expected_nvr_string) {
	$expected_nvr_string=~/^(.+)-([^-]+)-([^-]+)$/;
	if (not defined($1) or $1 ne $expected_name) {
	    warn "CheckNVR: $expected_name: mismatch with expected nvr: $expected_nvr_string\n";
	    return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
	} else {
	    ($expected_version,$expected_release)=($2, $3);
	}
    }
    my ($provided_name,$provided_version,$provided_release);
    my $provided_nvr_string=$converter_opt->{-name2spec_provided_nvr};
    if ($provided_nvr_string and $provided_nvr_string=~/^(.+)-([^-]+)-([^-]+)$/) {
	($provided_name,$provided_version,$provided_release)=($1, $2, $3);
    }

    unless ($provided_name) {
	my $given_spec=$spec;
	$given_spec=$parent if $opt_use_parent and $parent;
	#($provided_name,$provided_version,$provided_release)=$resourceobj->get_nvr($given_spec);
	my $mainsec=$given_spec->main_section;
	$provided_name=$mainsec->get_tag('Name');
    }
   
    if ($expected_name) {
	if ($expected_name ne $provided_name and $provided_name!~/\%/) {
	    warn "CheckNVR: $expected_name: name mismatch (--no-check-nvr will disable this check):\n";
	    warn "Expected: $expected_name\n";
	    warn "Reported: $provided_name\n";
	    return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
	}
	return RPM::Source::TransformContainer::TRANSFORMATION_OK unless defined $expected_version and defined $expected_release;
	
	my $expected_vr=$expected_version.'-'.$expected_release;
	my $provided_vr=$provided_version.'-'.$provided_release;
	if ($expected_vr ne $provided_vr  and $provided_vr!~/\%/) {
	    warn "CheckNVR: $expected_name: version mismatch (--no-check-nvr will disable this check):\n";
	    warn "Expected: $expected_vr\n";
	    warn "Reported: $provided_vr\n";
	    return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
	}
    }

    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;

=pod

=head1	CLI EXPORT SYNOPSIS

[B<--check-nvr | --no-check-nvr>]

=head1	CLI EXPORT OPTIONS

=item	[B<--check-nvr>]

Die if name and version-release of src.rpm and git repository spec file does not match.
Conversely, B<--no-check-nvr> allows to skip git repository version check.

=back

=cut

