package RPM::Source::Transformation::Factory::Datapath;

use strict;
use warnings;

# no use RPM::Source::Transformation::Resource::Datapath;
# it is a slave factory which is required at Resource::Datapath.
use RPM::Source::Transformation::Factory;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=0;

sub transformations {
    return (
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_SRPM_REINIT_FALLBACK,
	    # used in CheckIfChangedPlayer
	    -name => 'datapath reinit fallback',
	    -opt_handler_sub=>\&opt_handler_reinit_name_fallback),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_SRPM_AT_BEGIN,
	    # used in CheckIfChangedPlayer
	    -name => 'init copy path',
	    -opt_handler_sub=>\&opt_handler_init_copypath_AT_BEGIN),
	);
}

sub opt_handler_init_copypath_AT_BEGIN {
    my (undef,$spec,undef,)=@_;
    ## TODO: private access!!!
    $spec->{-copypath}=[@RPM::Source::Transformation::Resource::Datapath::patchdir],
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub just_for_no_warnings {
    @RPM::Source::Transformation::Resource::Datapath::patchdir;
}

sub opt_handler_reinit_name_fallback {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $oldname=$resourceobj->datapath_get_name();
    return RPM::Source::TransformContainer::TRANSFORMATION_OK if (defined($oldname) and $oldname ne '');
    # due to %fontpkg, %forgemeta, etc the name is not set in prepare_resource; try set it forcefully
    # at least it will help to load reinit hook to fix that package
    my ($source_name,)=$spec->_get_nevr();
    if (!defined($source_name)) {
	warn 'ERROR: Datapath: main package name not found. Please, define %name from command line.';
	return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
    }
    $resourceobj->datapath_set_name($source_name);
    print STDERR 'Datapath: datapath_set_name to ',$source_name,"\n" if $verbose;
    my $macros=$spec->macros;
    if (!$macros->get_raw('name')) {
	$macros->set_raw('name',$source_name);
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}


1;

__END__
