package RPM::Source::Transformation::Factory::Flags;

use strict;
use warnings;
use Carp;

use RPM::Source::Transformation::Resource::FlagLoader;
use RPM::Source::Transformation::Resource::Buildreq;
use RPM::Source::Transformation::Factory;

our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=0;

&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('filter-requires.txt',FLAG_RESOURCE_LIST_AS_ARRAY);
&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('filter-provides.txt',FLAG_RESOURCE_LIST_AS_ARRAY);
&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('add_findreq_skiplist.txt',FLAG_RESOURCE_LIST_AS_ARRAY);
&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('add_provides.txt',FLAG_RESOURCE_LIST_AS_ARRAY);
&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('add_requires.txt',FLAG_RESOURCE_LIST_AS_ARRAY);
&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('add_buildreq.txt',FLAG_RESOURCE_LIST_AS_ARRAY);


sub transformations {
    return RPM::Source::Transformation::OptHandlerWrapper->new(
	-priority => PRIORITY_PRECONVERT,
	-name => 'apply_flags',
	-opt_handler_sub=>\&opt_handler);
}

sub opt_handler {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;


    my $mainsec=$spec->main_section;
    

    my $filter_requires=$resourceobj->get_flagdb('filter-requires.txt');
    my $filter_provides=$resourceobj->get_flagdb('filter-provides.txt');
    my $add_findreq_skiplist=$resourceobj->get_flagdb('add_findreq_skiplist.txt');
    my $add_provides=$resourceobj->get_flagdb('add_provides.txt');
    my $add_requires=$resourceobj->get_flagdb('add_requires.txt');
    #my $=$resourceobj->get_flagdb('.txt');

    if ($filter_requires) {
	foreach my $i (map{&__perl_dep2filter($_)} @$filter_requires) {
	    $mainsec->verbatim_exclude_and_unshift('%filter_from_requires /^'.$i."/d\n");
	}
	print STDERR "Flags: filter_from_requires: ",join(', ',@$filter_requires),"\n" if $verbose;
    }
    if ($filter_provides) {
	foreach my $i (map{&__perl_dep2filter($_)} @$filter_provides) {
	    $mainsec->verbatim_exclude_and_unshift('%filter_from_provides /^'.$i."/d\n");
	}
	print STDERR "Flags: filter_from_provides: ",join(', ',@$filter_provides),"\n" if $verbose;
    }
    if ($add_findreq_skiplist) {
	foreach my $i (map{&__perl_vendormacrosize($_)} @$add_findreq_skiplist) {
	    $mainsec->verbatim_exclude_and_unshift('%add_findreq_skiplist '.$i."\n");
	}
	print STDERR "Flags: add_findreq_skiplist: ",join(', ',@$add_findreq_skiplist),"\n" if $verbose;
    }

    if ($add_provides) {
	$mainsec->verbatim_exclude_and_push('Provides: '.join(' ',@$add_provides)."\n");
	print STDERR "Flags: add_provides: ",join(', ',@$add_provides),"\n" if $verbose;
    }

    if ($add_requires) {
	$mainsec->verbatim_exclude_and_push('Requires: '.join(' ',@$add_requires)."\n");
	print STDERR "Flags: add_requires: ",join(', ',@$add_requires),"\n" if $verbose;
    }

    my $add_buildreq=$resourceobj->get_flagdb('add_buildreq.txt');
    if ($add_buildreq) {
	my $buildreq=$resourceobj->buildreq;
	$buildreq->add_buildreq(@$add_buildreq);
	print STDERR "Flags: add_buildreq: ",join(', ',@$add_buildreq),"\n" if $verbose;
    }
    #my $=$resourceobj->get_flagdb('.txt');

    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub __perl_dep2filter {
    my $dep=shift;
    $dep=~s![/()]!.!g;
    return $dep;
}

sub __perl_vendormacrosize {
    my $file=shift;
    # TODO
    return $file;
}

1;
__END__
# regular
q@%filter_from_provides '/^perl\(Perinci\\\\\/CmdLine\\\\\/Any\\\\\/Lumped\\\\\.pm\)/!d'@
# with \(\) group
@%filter_from_provides '/^perl\(\\\\\(Acme\\\\\|Alt\\\\\|Perinci\\\\\)\\\\\//!d'@
   when inserted in spec, it will be like %filter_from_provides '/^perl\(.*Bundle\\\//!d'
