package RPM::Source::Transformation::Factory::GroupTranslation;

use strict;
use warnings;
# DEPENDS on @datapath and @patchdir resolution.
use RPM::Source::Transformation::Resource::Datapath;
use RPM::Source::Transformation::Factory;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $VERSION=0.001;

#our $verbose=1;
my %GROUP_TRANSLATION_TABLE;
my $opt_group_tr_cmdline;

# TODO: drop or inherit
sub new {
    my $class = shift;
    my $self = {
	@_
    };
    bless $self, $class;
    return $self;
}

sub print_longopt {
    print "
  Group Editing options:
	--group-translate 'group translation'
";
}

our @LONGOPT=(
    'group-translate=s'   => \$opt_group_tr_cmdline,
);

sub process_options {
    my ($self)=@_;
    if ($opt_group_tr_cmdline) {
	my @tr= split (/\s*[\t\n,;|]+\s*/,$opt_group_tr_cmdline);
	while (@tr>1) {
	    my $key=shift @tr;
	    my $val=shift @tr;
	    $GROUP_TRANSLATION_TABLE{$key}=$val if $key and $val;
	}
    }
}

sub transformations {
    return RPM::Source::Transformation::OptHandlerWrapper->new(
	-priority => PRIORITY_PRECONVERT,
	-name => 'command-line group translation',
	-opt_handler_sub=>\&opt_handler);
}

sub opt_handler {
    my ($self,$spec,undef,$opt,$resourceobj)=@_;

    # looking for default group translation
    my %local_group_translation_table;
    map {&__::GTransform::__load_group_tr_file(\%local_group_translation_table, $_)} 
        grep {-e $_} $resourceobj->find_named_filesystem_resources('groups','');
    map {$local_group_translation_table{$_}=$GROUP_TRANSLATION_TABLE{$_}} keys %GROUP_TRANSLATION_TABLE
	if %GROUP_TRANSLATION_TABLE;

    &__::GTransform::__subst_groups($spec, \%local_group_translation_table) if %local_group_translation_table;
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}


# private Transform modules
package __::GTransform;

use strict;
use warnings;

sub __subst_groups {
    my ($spec, $transltbl)=@_;
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	    my $group=$sec->get_tag('Group');
	    $sec->set_tag('Group', $transltbl->{$group}) if $group and $transltbl->{$group};
	}
    }
}

sub __load_group_tr_file {
    my ($translation_table, $grouphintfile)=@_;
    return unless -e $grouphintfile;
    open my $grouphint, '<' , $grouphintfile || die "can't open $grouphintfile: $!";
    while (<$grouphint>) {
	chomp;
	s/\s*$//;
	next if /^#/;
	next if /^\s*$/;
	my @map=split(/\s*[\t|]\s*/,$_);
	warn "$grouphintfile: extra args: @map\n" if @map >2;
	$translation_table->{$map[0]}=$map[1] if $_;
    }
    close $grouphint;
}


1;
