package RPM::Source::Transformation::Factory::ParentI18NMerge;

use strict;
use warnings;
use Carp;

use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_POSTHOOKS,
	    -name => 'merge parent I18L Summary and description',
	    -opt_handler_sub=>\&opt_handler_merge_parent_I18L_Summary_and_description);
}

sub opt_handler_merge_parent_I18L_Summary_and_description {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    # TODO: move to Transform (see documentation)
    # merge language-translated summary and description
    if ($parent) {
	my $need_reset=0;
	foreach my $sec ($parent->get_sections()) {
	    my $t = $sec->get_type();
	    my $p = $sec->get_raw_package();
	    my $extra=$sec->get_extra_qualifier();
	    if ($t eq 'package') {
		my $newsec=0;
		foreach (@{$sec->get_bodyref()}) {
		    if (/^(Summary\(\S+\)):\s*(.+)/i) {
			my $tag=$1;
			my $val=$2;
			# lazy initialization
			$newsec=$spec->get_section($t,$p) if not $newsec and defined $newsec;
			if ($newsec) {
			    my $summary=$newsec->get_tag($tag);
			    $newsec->set_tag($tag,$val) unless $summary;
			}
		    }
		}
	    } elsif ($t eq 'description' and $extra) {
		my $newsec=$spec->get_section($t,$p,$extra);
		unless ($newsec) {
		    $newsec=$spec->get_section($t,$p);
		    next unless $newsec;
		    my $newsec_bodyref=$newsec->get_bodyref();
		    push @$newsec_bodyref, "\n" if $newsec_bodyref->[$#{$newsec_bodyref}] ne "\n";
		    push @$newsec_bodyref, grep {not /^\%(?:if|else|endif)/} @{$sec->get_bodyref()};
		    $need_reset=1;
		}
	    }
	}
	$spec->_reset_speclist() if $need_reset;
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;
