package RPM::Source::Transformation::Factory::RaiseRelease;

use strict;
use warnings;
use Cwd;
use Carp;

use RPM::Source::Transformation::Factory;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $VERSION=0.054;
our $verbose=1;

# opt'ified, seems to be not used
our ($opt_default_packager_policy,
     $opt_version,$opt_release,$opt_epoch,
     $opt_add_changelog,
);

# not optified yet, just reserved for future use;
my $opt_force_replace_version;

#### used in jppimport
our $packager;

# used in ALTLinuxBackport
# transform mode: parent is igonored at release increment,
our $force_transform_mode; # changelog is from origin, not from parent

our $nextrel='nmu';

#our $opt_ubt_mode='auto';
our $opt_ubt_mode='none';

# NOTE: $defaultchangelog is appenable default changelog.
# hooks can append entries to the default changelog
# whereas $changelog_entry will override any of them.
# NOTE: 2 possible entry points for default changelog:
# 1) new(CHANGELOG=>, 2) set_default_changelog/add_default_changelog
# TODO: rename CHANGELOG => -changelog in R::S::E? used in repocop
our ($opt_changelog_packager,$opt_changelog_entry,$opt_changelog_date);
our ($opt_changelog_use_raw_release,$opt_no_serial2epoch);
# end opt'ified
our $defaultchangelog='- ';
# rpmvercmp; do not change rashly, only for cross-spec-ing to foreign rpm
# API follows Source::Package::Comparators::Raw::*
our $version_comparator=\&__::Transform::__rpm_version_compare;

# optified
my $opt_refresh_timestamp;

# should not be optified; overrides supplied values.
my $usersetdefaultchangelog;

our %NEXTREL_METHOD=(
    'incr' => \&__::Transform::nextrel_incr,
    'incrfirst' => \&__::Transform::nextrel_incrfirst,
    'incrsecond' => \&__::Transform::nextrel_incrsecond,
    'qa' => \&__::Transform::nextrel_qa,
    'nmu' => \&__::Transform::nextrel_nmuadd,
    'nmuappend' => \&__::Transform::nextrel_nmuappend,
    'nmuadd' => \&__::Transform::nextrel_nmuadd,
    'none' => \&__::Transform::nextrel_none,
    );

our %INITREL_METHOD=(
    );

our %REL_METHOD_DESCR=(
    'incr' => 'increment release digits.',
    'incrfirst' => 'increment first component of release number, keeping the tail.',
    'incrsecond' => 'increment the last component of release number.',
    'qa' => 'add/increment .qaN',
#    'nmu' => '',
    'nmuappend' => 'append .1',
    'nmuadd' => 'append .1 for a single-component release or increment the second component.',
    'none' => 'do nothing.',
#    '' => '',
    );

sub _print_nextrel_methods {
    print "	Available next release methods: 
";
    my %main_name=map {$NEXTREL_METHOD{$_}=>$_} keys (%REL_METHOD_DESCR);
    my %synonym=map {$_=>[]} keys (%REL_METHOD_DESCR);
    foreach my $sname (sort keys(%NEXTREL_METHOD)) {
	my $description=$REL_METHOD_DESCR{$sname};
	my $mname=$main_name{$NEXTREL_METHOD{$sname}};
	push@{$synonym{$mname}}, $sname if $mname and not defined $description;
    }
    foreach my $sname (sort keys(%NEXTREL_METHOD)) {
	my $description=$REL_METHOD_DESCR{$sname};
	if (defined $description) {
	    print "\t\t$sname\t- $description\n";
	    my @synonyms = @{$synonym{$sname}};
	    print "\t\t\t  Synonyms: ".join(',',@synonyms)."\n" if @synonyms;
	}
    }
}

sub register_nextrel_method {
    my $in = {@_};
    my $name=$in->{-name};
    warn "register_nextrel_method: $name already registered" if $NEXTREL_METHOD{$name};
    my $nextrel=$in->{-nextrel_method};
    my $initrel=$in->{-initrel_method};
    $NEXTREL_METHOD{$name}=$nextrel if $nextrel;
    $INITREL_METHOD{$name}=$initrel if $initrel;
    $REL_METHOD_DESCR{$name}=$in->{-description};
    die "Oops: method registration for $name failed: no method" if not $nextrel and not $initrel;
}

sub print_longopt {
    print "
  Release options:
	--tag-packager-replace 'new value' set Packager: tag to a new value.
	  special values are: 
	    auto - use rpm's \%{packager} value
	    none - clear tag
	    parent - use parent's value of Packager:
	--tag-packager-default 'new value' - if the Packager: tag is not defined
	  set it to the new value. This option allows comma separated list
	  of multiple sources, for example, 'parent,A. R. <ar\@b.org>'
	--changelog '- changelog entry'
	--add-changelog  add changelog for current Epoch:Name-Version
	--epoch 'new value'
	--version 'version' - set new package version
	--release 'release' - if we need to adjust the package release
	--nextrel|--next-release-policy <policy>
	  known next release policies: 
	".join(', ',sort(keys %NEXTREL_METHOD))."
	  default policy is $nextrel.
	--refresh-timestamp update timestamp in existing changelog
	--ubt-mode=[never,auto,always] add %ubt to release. default is auto.
";
}

our @LONGOPT=(
    'add-changelog|add_changelog'   => \$opt_add_changelog,
    'ch|changelog=s'   => \$usersetdefaultchangelog,
    # changelog-override options
    'changelog-entry=s'	=> \$opt_changelog_entry,
    'changelog-packager=s'   => \$opt_changelog_packager,
    'changelog-date=s'	=> \$opt_changelog_date,
    'no-serial2epoch' => \$opt_no_serial2epoch,
    'epoch=s'   => \$opt_epoch,
    'nextrel|next-release-policy=s' => \$nextrel,
    'refresh-timestamp' => \$opt_refresh_timestamp,
    'release=s'	=> \$opt_release,
    'version=s'	=> \$opt_version,
    'tag-packager-default=s' => \$opt_default_packager_policy,
    'tag-packager-replace=s' => \$packager,
    'packager=s' => \$packager,
    'ubt-mode=s' => \$opt_ubt_mode,
);

sub register_messages {
    my ($self,$extopt)=@_;
    ### TODO: use some fixed keys hash here
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-packager => $packager,
	-default_packager_policy => $opt_default_packager_policy,
	-default_changelog => $defaultchangelog,
	-add_changelog => $opt_add_changelog,
	-version => $opt_version,
	-release => $opt_release,
	-epoch => $opt_epoch,
	-nextrel => $nextrel,
	-force_transform_mode => $force_transform_mode,
	-refresh_timestamp => $opt_refresh_timestamp,
	-changelog_entry => $opt_changelog_entry,
	-changelog_packager => $opt_changelog_packager,
	-changelog_date => $opt_changelog_date,
	-ubt_mode => $opt_ubt_mode,
	});
}

sub transformations {
    my @transformations;
    push @transformations,
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_SRPM_AT_BEGIN,
	    -name => 'init default changelog',
	    -opt_handler_sub=>\&opt_handler_init_default_changelog_AT_BEGIN);
    push @transformations,
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_SRPM_AT_BEGIN,
	    -name => 'convert serial to epoch',
	    -opt_handler_sub=>\&opt_handler_convert_serial_to_epoch_AT_BEGIN)
	if !$opt_no_serial2epoch;
    push @transformations,
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_SRPM_AT_BEGIN,
	    -name => '%%ubt macros support',
	    -opt_handler_sub=>\&opt_handler_ubt_support_AT_BEGIN),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_RAISE_RELEASE,
	    # Careful! Used in CheckIfChangedPlayer!
	    -name => 'raise release',
	    -opt_handler_sub=>\&opt_handler);
    return @transformations;
}

sub opt_handler_init_default_changelog_AT_BEGIN {
    my (undef,$spec,$parent,$opt)=@_;
    my $def_changelog=$opt->{-default_changelog};
    if ($usersetdefaultchangelog) {
	$spec->set_default_changelog($usersetdefaultchangelog);
    } elsif ($def_changelog) {
	$spec->set_default_changelog($def_changelog);
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub opt_handler_ubt_support_AT_BEGIN {
    my (undef,$spec,$parent,$opt)=@_;
    my $macros=$spec->macros();
    $macros->set_raw('ubt','%__ubt_branch_prefix%__ubt_branch_id%__ubt_branch_suffix');
    if (!$DistroMap::default_destbranch or $DistroMap::default_destbranch eq 'sisyphus') {
	$macros->set_raw('__ubt_branch_prefix','.');
	$macros->set_raw('__ubt_branch_id', 'S1');
	$macros->set_raw('__ubt_branch_suffix','%nil');
    } elsif ($DistroMap::default_destbranch eq 'p8') {
	$macros->set_raw('__ubt_branch_prefix','.');
	$macros->set_raw('__ubt_branch_id', 'M80P');
	$macros->set_raw('__ubt_branch_suffix','.1');
    } 
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub opt_handler_convert_serial_to_epoch_AT_BEGIN {
    my (undef,$spec,$parent,$opt)=@_;
    foreach my $section ($spec->get_sections()) {
	my $type=$section->get_type;
	next unless $type eq 'package';
	my $pkg=$section->get_raw_package;
	my $is_mapped=0;
	$section->map_body(sub {
	    if (s/^(\s*)serial:(\s*)(\S.*)/${1}Epoch:$2$3/i) {
		if ($pkg eq '') {
		    my $raw_value=$3;
		    $raw_value=~s/\s*$//;
		    $spec->macros->del_raw('serial');
		    $spec->macros->set_raw('epoch',$raw_value);
		}
	    } elsif (/^Requires:/i) {
		s,\%\{?serial:\%serial:\},%{?epoch:%epoch:},g;
		s,\%\{?serial:\%\{serial\}:\},%{?epoch:%{epoch}:},g;
		s,\%\{serial\},%{epoch},g;
		s,\%serial:,%epoch:,g;
	    }
	});

    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

# there is a reference to this subroutine in repocop; TODO: refactor it
sub opt_handler {
    my (undef,$spec,$parent,$opt)=@_;
    &__::Transform::__set_packager	($spec,$parent,$opt->{-default_packager_policy},$opt->{-packager});
    &__::Transform::__refresh_timestamp($spec) if $opt->{-refresh_timestamp};

    my $nextrel=$opt->{-nextrel};
    my $version=$opt->{-version};
    my $set_epoch=$opt->{-epoch};
    my $we_change_evr=(
	$nextrel ne 'none'
	or defined($version)
	or defined($set_epoch)
	or defined($opt->{-release})
	);
    if ($we_change_evr) {
	# used in version comparator
	my ($oldname,$newname);
	$oldname=$parent->main_section->get_tag('Name') if $parent;
	$newname=$spec->main_section->get_tag('Name');
	my @old_nevr=$spec->_get_nevr();
	my @parent_nevr=@old_nevr;
	@parent_nevr=$parent->_get_nevr() if $parent;
	my $new_epoch=&__::Transform::__set_epoch($spec,$parent,$version,$oldname,$newname,$set_epoch,$opt->{-rename});
	my @new_nevr=@old_nevr; $new_nevr[1]=$new_epoch;
	&__::Transform::__set_version($spec,$parent,$version,\@new_nevr) if defined $version;
	&__::Transform::__increment_release($spec,$parent,$oldname,$newname,$nextrel,$opt,\@new_nevr,\@parent_nevr);
        # TODO: update %release, %version, %epoch macros to final evaled values.
        # TODO: a solution for 'fixes ahead' - %define release, %define version

	&__::Transform::__rename_sources_and_patches($spec,$parent,\@old_nevr,\@new_nevr);
    }
    if ($we_change_evr or $opt_add_changelog) {
	&__::Transform::__set_changelog	($spec,$parent,$opt->{-changelog_entry},$opt->{-changelog_packager},$opt->{-changelog_date},$opt->{-force_transform_mode});
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

# private Transform modules
package __::Transform;

use strict;
use warnings;
use File::Basename;
use RPM::Vercmp;
use File::Temp qw/tempfile tempdir/;

# TODO:
sub __rpm_version_compare {
    my ($oldname,$newname,$oldversion,$newversion)=@_;
    return RPM::Vercmp::evrcmp($oldversion,$newversion);
}


sub __refresh_timestamp {
    my ($spec) = @_;
    my $changelogsec=$spec->get_section('changelog');
    die "Refresh timestamp: no changelog found" unless $changelogsec;
    my $date=`LC_TIME=C date '+%a %b %d %Y'`;
    chomp $date;
    warn "changelog update failed" unless $changelogsec->get_bodyref
	->[1]=~s/^\* \w\w\w \w\w\w \d\d 20\d\d /* $date /;
}

sub __set_packager {
    my ($spec,$parent,$default_packager_policy,$packager) = @_;
    my $mainsec=$spec->main_section();
    my $packager_tag=$mainsec->get_tag('Packager');
    if ($packager) {
	if ($packager eq 'auto') {
	    $packager=$spec->macros()->macro_subst('%{packager}');
	} elsif ($packager eq 'none') {
	    $mainsec->clear_tag('Packager');
	} elsif ($packager eq 'parent') {
	    die "--packager=parent but there is no parent\n" unless $parent;
	    my $candidate=$parent->main_section()->get_tag('Packager');
	    if ($candidate) {
		$mainsec->set_tag('Packager',$candidate);
	    } else {
		$mainsec->clear_tag('Packager');
	    }
	} else {
	    $mainsec->set_tag('Packager',$packager);
	}
    } elsif (! $packager_tag and $default_packager_policy) {
	foreach my $policy (split(/,/,$default_packager_policy)) {
	    if ($policy eq 'parent') {
		next unless $parent;
		my $candidate=$parent->main_section()->get_tag('Packager');
		if ($candidate) {
		    $mainsec->set_tag('Packager',$candidate);
		    last;
		}
	    } elsif ($policy eq 'auto') {
		my $candidate=$spec->macros()->macro_subst('%{packager}');
		if ($candidate) {
		    $mainsec->set_tag('Packager',$candidate);
		    last;
		}
	    } elsif ($policy =~ m'\@') {
		# default packager
		$mainsec->set_tag('Packager',$policy);
		last;
	    }
	}
    }
}

sub __set_changelog {
    my ($spec,$parent,$changelog_entry,$changelog_packager,$changelog_date,$force_transform_mode)=@_;
    my $changelogsec=$spec->add_section('changelog');
    #die "Oops! changelog section not found" unless $changelogsec;
    if ($parent && !$force_transform_mode) {
	$changelogsec->set_body($parent->get_section('changelog')->get_bodyref());
    }
    # old hack to re-split lines against broken add_changelog entries
    $changelogsec->set_body($changelogsec->get_bodyref());

    my $raw_release=$spec->main_section->get_tag('Release',RAW=>1);
    my $changelog_use_raw_release=$opt_changelog_use_raw_release;
    $changelog_use_raw_release=1 if $raw_release=~/\%\{?ubt\}?\b/;
    my @args;
    push @args, 'ENTRY' => $changelog_entry if $changelog_entry;
    push @args, 'PACKAGER' => $changelog_packager if $changelog_packager;
    push @args, 'DATE' => $changelog_date if $changelog_date;
    push @args, 'RELEASE' => $raw_release if $changelog_use_raw_release;

    $spec->add_changelog(@args);
}

sub __epoch_val {
    my ($epoch,$serial) =@_;
    if (defined $serial) {
	if (defined $epoch) {
	    return ($epoch > $serial ) ? $epoch:$serial;
	} else {
	    return $serial;
	}
    } elsif (defined $epoch) {
	return $epoch;
    } else {
	return;
    }
}

sub __undef {
    return defined $_[0] ? $_[0] : 'undef';
}

sub __set_epoch {
    my ($spec,$parent,$version,$oldname,$newname,$epoch,$opt_rename)=@_;
    my $specmain=$spec->main_section();
    my $oldepoch = $specmain->get_tag('Epoch');
    my $oldserial = $specmain->get_tag('Serial');
    my $oldval= &__epoch_val($oldepoch,$oldserial);
    my $parentver;
    if ($parent and not defined $epoch) {
	my $parentmain=$parent->main_section();
	$parentver=$parent->main_section()->get_tag('Version');
	my $parentepoch = $parentmain->get_tag('Epoch');
	my $parentserial = $parentmain->get_tag('Serial');
	my $parentval= &__epoch_val($parentepoch,$parentserial);
	if (! defined $oldval || (defined $parentval and $parentval > $oldval)) {
	    if (defined $parentval) {
		$epoch=$parentval;
		$oldepoch=$parentval;
	    }
	    $oldval=$parentval;
	    print STDERR "note: picked up epoch $parentval from parent\n" if defined($parentval) and $verbose;
	}
    }
    my $specver=$spec->main_section()->get_tag('Version');
    $parentver//=$specver;
    $specver=$version if defined($version);
    my $vercmp = &$RPM::Source::Transformation::Factory::RaiseRelease::version_comparator($oldname, $newname, $parentver,$specver);
    print STDERR 'DEBUG: RaiseRelease: epoch: check ver: oldname=',&__undef($oldname),", newname=$newname, parentver=$parentver, specver=$specver cmp=$vercmp\n" if $verbose>2;
    if ($vercmp > 0) {
	print STDERR "DEBUG: downgrade: oldname=$oldname, newname=$newname, parentver=$parentver,specver=$specver cmp=$vercmp\n" if $verbose>1;
	if ($opt_rename and $opt_rename ne $newname) {
	    print STDERR "note: downgrade detected but ignored due to rename to $opt_rename\n" if $verbose;
	} else {
	    if (defined($oldval)) {
		$epoch=++$oldval;
	    } else {
		$epoch=1;
	    }
	    print STDERR "note: downgrade detected; increasing epoch to $epoch\n" if $verbose;
	}
    }
    if (defined $epoch) {
	if ($epoch eq 'none') {
	    $specmain->exclude_body(qr'^Epoch:');
	    $spec->macros()->del_raw('epoch');
	    $epoch=undef;
	} else {
	    $specmain->set_tag('Epoch',$epoch);
	    $spec->macros()->set_raw('epoch',$epoch);
	}
	$specmain->exclude_body(qr'^Serial:');
	$spec->macros()->del_raw('serial');
    }

    # adjust epoch in strict dependencies
    if (defined $epoch) {
	foreach my $sec ($spec->get_sections) {
	    my $type=$sec->get_type;
	    next unless $type eq 'package';
	    $sec->map_body(
		sub {
		    if (/^Requires:/i) {
			s,\s+=\s+\%\{version\}-\%\{release\}, = %{?epoch:%epoch:}%{version}-%{release},g;
			s,\s+=\s+\%version-\%release, = %{?epoch:%epoch:}%version-%release,g;
		    }
		});
	}
    }
    return $epoch;
}

sub __set_version {
    my ($spec,$parent,$version,$new_spec_nevr)=@_;
    return unless defined $version;
    my $mainsec=$spec->main_section();
    my $replace_tag=1;
    my $specver=$mainsec->get_tag('Version', RAW => 1);

    if ($specver =~ /\%.+\%/) {
	print STDERR "WARNING:".$mainsec->get_tag('Name', RAW => 0).": version $specver concatenates macros! will be overriden\n";
    } elsif ($specver =~ /^\%\{?(\S+)\}?$/) { # one macro
	my $vermacros=$1;
	my $replaced=0;
	$mainsec->map_body(sub{
	    $replaced||=s/^(\s*\%(?:define|global)\s+$vermacros\s+)\S+/$1$version/;
			   });
	if ($replaced) {
	    $spec->macros->set_raw($vermacros,$version);
	    $replace_tag=0 unless $opt_force_replace_version;
	} else {
	    print STDERR "WARNING:can't find the definition of $vermacros for ".$mainsec->get_tag('Name', RAW => 0)."\n";
	}
    }
    if ($replace_tag) {
	$mainsec->set_tag('Version', $version);
	$spec->macros->set_raw('version',$version);
    }
    $new_spec_nevr->[2]=$version;
    return;
}

sub __increment_release() {
    my ($spec,$parent,$oldname,$newname,$nextrel_policy,$opt,$new_spec_nevr,$old_nevr)=@_;
    my ($forced_value,$force_transform_mode,$ubt_mode)=($opt->{-release},$opt->{-force_transform_mode},$opt->{-ubt_mode});
    if (!defined ($forced_value) && !$force_transform_mode) {
	my $vercmp = &$RPM::Source::Transformation::Factory::RaiseRelease::version_comparator($oldname, $newname, $old_nevr->[2],$new_spec_nevr->[2]);
	if ($vercmp < 0) {
	    # upgrade;
	    $forced_value='alt1';
	    my $initrel_method=$INITREL_METHOD{$nextrel_policy};
	    $forced_value=&$initrel_method($spec,$parent) if $initrel_method;
	    if ($ubt_mode and $ubt_mode ne 'none') {
		if ($ubt_mode eq 'force' or $ubt_mode eq 'always') {
		    $forced_value.='%ubt' if $forced_value !~/\%\{?ubt\}?$/;
		} elsif ($ubt_mode eq 'auto') {
		    my $oldparentrel=$parent->main_section->get_tag('Release', RAW=>1) if $parent;
		    my $oldrel=$spec->main_section->get_tag('Release', RAW=>1);
		    if (defined $oldparentrel and $oldparentrel=~/\%\{?ubt\}?$/ or
			defined $oldrel and $oldrel=~/\%\{?ubt\}?$/) {
			$forced_value.='%ubt' if $forced_value !~/\%\{?ubt\}?$/;
		    }
		} elsif ($ubt_mode eq 'never') {
		    $forced_value =~ s,\%\{?ubt\}?\b,,;
		} else {
		    die "RaiseRelease: bad value $ubt_mode for $ubt_mode";
		}
	    }
	    print STDERR "note: increment release: version upgrade: forced_value=$forced_value\n" if $verbose>1;
	} elsif ($vercmp>0) {
	    # downgrade;
	    # just increment release normally, it usually reasonable.
	    print STDERR "note: increment release: version downgrade.\n" if $verbose>1;
	}
    } 
    my $mainsec=$spec->main_section();
    my $macros=$spec->macros();
    my $bodyref=$mainsec->get_bodyref();
    for (my $i=0; $i<@$bodyref; $i++) {
	#if ($bodyref->[$i]=~/^(release:\s*)(\S+)(.+)/si) {
	# Release %macro arg contains spaces :(
	if ($bodyref->[$i]=~/^(release:\s*)(\S.+)$/si) {
	    my ($pre,$rel)=($1,$2);
	    $rel=~s/(\s*)$//;
	    my $post=$1;
	    $post="\n" unless $post;
	    # hack around;
	    # recurrent release like %define release alt1; Release %release.1.2.3
	    $rel=$macros->get_raw('release') if $rel =~ /\%\{release\}/ or $rel =~ /\%release\b/;
	    print STDERR "debug:before nextrel pre=[$pre] rel=[$rel]\n" if $verbose>2;
	    print STDERR "debug:forced_value=$forced_value\n" if $forced_value and $verbose>2;
	    $rel=$forced_value ? $forced_value : &__::Transform::__raw_increment_release($nextrel_policy, $rel,$ubt_mode,$spec,$parent);
	    $bodyref->[$i]=$pre.$rel.$post;
	    die "FATAL: no endline: [$pre][$rel][$post]" unless $bodyref->[$i]=~/\n$/;
	    $macros->set_raw('release',$macros->macro_subst($rel));
	    $new_spec_nevr->[3]=$rel;
	    print STDERR "debug:after nextrel pre=[$pre] rel=[$rel]\n" if $verbose>2;
	}
    }
    $mainsec->set_body($bodyref);
}


sub __rename_sources_and_patches() {
    my ($spec,$parent,$old_nevr,$new_nevr)=@_;
    ## HACK to rename sources along with new release. 
    # DO not run in GEAR mode.
    return if $spec->{GEARDIR};

    my $old_vr=$old_nevr->[2].'-'.$old_nevr->[3];
    my $new_vr=$new_nevr->[2].'-'.$new_nevr->[3];

    print STDERR "__rename_sources_and_patches: old_vr=$old_vr new_vr=$new_vr\n" if $verbose>3;

    if ($spec->{SOURCERPM} and -d $spec->{SOURCEDIR} and $old_vr and $new_vr) {
	foreach my $file (glob "$spec->{SOURCEDIR}/*") {
	    next unless -f $file;
	    my $basename=basename($file);
	    my $copy=$basename;
	    $copy=~s/\Q$old_vr\E/$new_vr/;
	    if ($copy ne $basename) {
		system ('cp',$file,"$spec->{SOURCEDIR}/$copy")==0 or die "cp $file $spec->{SOURCEDIR}/$copy failed!";
	    }
	}

	my $prepsec=$spec->get_section('prep');
	#%setup -q -n %name-%version-%release
	if ($prepsec and $prepsec->match_body(qr'^\s*\%setup.+-n\s+.+-\%\{?version\}?-\%\{?release\}?')) {
	    my $repack=0;
	    foreach my $file (glob "$spec->{SOURCEDIR}/*") {
		next unless -f $file and $file=~/\.tar(\.(?:gz|bz2))?$/;
		my $arj='';
		if ($1) {
		    $arj='z' if ($1 eq '.gz');
		    $arj='j' if ($1 eq '.bz2');
		}
		my $tmpdir = tempdir( "tmp-unpack-source.XXXXXXXX", TMPDIR => 1, CLEANUP=>1);
		die "can't create temporary directory!" unless $tmpdir;
		#my $curdir=cwd;
		#chdir $tmpdir;
		system('tar','x'.$arj.'f',$file,'-C',$tmpdir)==0 or die 'tar x'.$arj."f $file -C $tmpdir failed!";
		my ($dir)=glob "$tmpdir/*";
		if ( -d $dir) {
		    my $basename=basename($dir);
		    my $copy=$basename;
		    $copy=~s/\Q$old_vr\E/$new_vr/;
		    if ($copy ne $basename) {
			system ('mv','-f',$dir,"$tmpdir/$copy")==0 or die "mv $dir $tmpdir/$copy failed!";
			system ('tar','c'.$arj.'f',$file,'-C',$tmpdir,$copy)==0 or die 'tar c'.$arj."f $file -C $tmpdir $copy failed!";
			$repack=1;
		    }
		}
	    }
	    unless ($repack) {
		print STDERR "WARNING: found \%setup -q -n ...-\%version-\%release\n";
	    }
	}

	# hack: fix version in patches
	my $mainsec=$spec->main_section();
	my $oldver=$old_nevr->[2];
	my $oldrel=$old_nevr->[3];
	my $newver=$new_nevr->[2];
	my $newrel=$new_nevr->[3];
	# vr's are already renamed; so no need to touch release

	print STDERR "__rename_sources_and_patches: oldver=$oldver oldrel=$oldrel newver=$newver newrel=$newrel\n" if $verbose>3;
	$mainsec->map_body(sub {
	    return unless /^((?:source|patch)[^:]*:\s+)(\S+)(\s*)$/i;
	    my $prefix=$1;
	    my $path=$2;
	    my $suffix=$3;
	    my $basename=basename($path);
	    my $url=substr($path,0,length($path)-length($basename));
	    # basename(macro_subst(basename()) helps for SourceN: %{someurl}file.tar
	    return if -e "$spec->{SOURCEDIR}/".basename($spec->macros()->macro_subst($basename));

	    my $new_filename=$basename;
	    return if &__check_if_exist_new_filename($spec,\$new_filename,$basename, $prefix.$url, $suffix);
	    $new_filename=~s/\%\{?version\}?/$oldver/;
	    return if &__check_if_exist_new_filename($spec,\$new_filename,$basename, $prefix.$url, $suffix);
	    $new_filename=~s/\%\{?version\}?/$newver/;
	    return if &__check_if_exist_new_filename($spec,\$new_filename,$basename, $prefix.$url, $suffix);
	    $new_filename=~s/\Q$oldver\E/$newver/;
	    $new_filename=~s/\Q$oldrel\E/$newrel/;
	    return if &__check_if_exist_new_filename($spec,\$new_filename,$basename, $prefix.$url, $suffix);
	    warn "WARNING: setting new version: can't find $prefix$path. Pray for the best!";
			   });

    }
}

sub __check_if_exist_new_filename {
    my ($spec,$new_filenameref,$basename, $prefix,$suffix)=@_;
    my $expanded_name="$spec->{SOURCEDIR}/".basename($spec->macros()->macro_subst($$new_filenameref));
    if (-e $expanded_name) {
	$_=$prefix.$$new_filenameref.$suffix;
	return 1;
    }
    $$new_filenameref=~s/\.(?:gz|bz2|xz)$//;
    my $new_expanded_name=$expanded_name;
    $new_expanded_name=~s/\.(?:gz|bz2|xz)$//;
    if (-e $expanded_name.'.gz') {
	&run('gunzip',$expanded_name.'.gz');
    } elsif (-e $expanded_name.'.bz2') {
	&run('bunzip',$expanded_name.'.bz2');
    } elsif (-e $expanded_name.'.xz') {
	&run('unxz',$expanded_name.'.xz');
    }
    if (-e $new_expanded_name) {
	$_=$prefix.$$new_filenameref.$suffix;
	return 1;
    }

    $$new_filenameref=$basename;
    return 0;
}

sub __raw_increment_release {
    my ($nextrel_policy,$rel,$ubt_mode,$spec,$parent)=@_;
    $nextrel_policy||= 'incr';
    my $nextrel_method=$NEXTREL_METHOD{$nextrel_policy};
    unless ($nextrel_method) {
	&RPM::Source::Transformation::Factory::RaiseRelease::_print_nextrel_methods();
	die "Unknown next release policy: $nextrel_policy\n";
    }
    my $ubt='';
    $ubt=$1 if $rel =~ s/(\%\{?ubt\}?)$//;
    if ($ubt_mode) {
	$ubt//='%ubt' if $ubt_mode eq 'force' or $ubt_mode eq 'always';
	$ubt='' if $ubt_mode eq 'never' or $ubt_mode eq 'none';
    }
    my $newrel = &{$nextrel_method}($rel,$spec,$parent);
    $newrel||= $rel.".1";
    return $newrel.$ubt;
}


sub run {
    print STDERR "running: ", join(' ',@_),"\n" if $verbose > 4;
    system(@_)==0 or die "command failed: ".join(' ',@_);
}

#----------------------------------------------------------
#########################################
# private:  release update policies
#########################################
sub nextrel_incr {
    my ($rel)=@_;
    my $newrel;
    if ($rel=~/^((?:\D+)?)(\d+).*$/) {
        my $relprefix=$1;
        my $digitrel=$2;
        $newrel = $relprefix.++$digitrel;
    }
    return $newrel;
}

sub nextrel_incrfirst {
    my ($rel)=@_;
    my $newrel;
    if ($rel=~/^((?:\D+)?)(\d+)(.*)$/) {
        my $relprefix=$1;
        my $digitrel=$2; my $tail=$3;
        $newrel = $relprefix.++$digitrel.$tail;
    }
    return $newrel;
}

sub nextrel_incrsecond {
    my ($rel)=@_;
    my $newrel;
    if ($rel=~s/\.(\d+)$//) {
	my $count=$1;
	$newrel = $rel.".".($count+1);
    } else {
	$newrel = $rel.'.1';
    }
    return $newrel;
}

sub nextrel_qa {
    my ($rel)=@_;
    my $newrel;
    if ($rel=~s/\.qa(\d+)(?:\.(?:qa\d+|1))*$//) {
	$newrel = $rel.".qa".($1+1);
    } else {
	$newrel = $rel.'.qa1';
    }
    return $newrel;
}

sub nextrel_nmuappend {
    my ($rel)=@_;
    return $rel.".1";
}

sub nextrel_nmuadd {
    my ($rel)=@_;
    my $newrel;
    if ($rel=~s/\.(\d+)(?:\.\d)*$//) { # like, alt5.1.1.1.1.1
	$newrel = $rel.".".($1+1);
    } elsif ($rel=~s/(\d+)([(?:\D+)?\%[\w\d\._]+)$//) {
	# hack around %ubt: grenka: Тупая пересборка с повышением релиза. Получаем в итоге "горе от ума". Girar-nmu инкрементирует релиз у ubt-пакетов перед макросом %ubt и на выходе получаем, что новая версия не старше предыдущей, потому что буква S больше чем 1.
	$newrel = $rel.($1+1).$2;
    } else {
	$newrel = $rel.".1";
    }
    return $newrel;
}

sub nextrel_none {
    my ($rel)=@_;
    return $rel;
}

1;
