package RPM::Source::Transformation::Factory::TryRPMBuildPrep;

use strict;
use warnings;
use Carp;
use File::Temp;

use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Transformation::Factory;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $VERSION=0.004;
# $verbose=0;
our $verbose=1;
our $opt_try_rpmbuild_prep=0;

sub register_messages {
    my ($self,$extopt)=@_;
    ### TODO: use some fixed keys hash here
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-opt_try_rpmbuild_prep => $opt_try_rpmbuild_prep,
	-opt_try_rpmbuild_stagearg => '-bp',
# TODO:
#	-opt_try_rpmbuild_use_hasher => undef,
	-try_rpmbuild_prep_tmpdir => undef,
	-try_rpmbuild_prep_sourcetree => undef,
    });
}

sub transformations {
    return (
	RPM::Source::Transformation::OptHandlerWrapper->new(
	-priority => PRIORITY_TRY_RPMBUILD_PREP_EXEC,
	-name => 'exec_rpmbuild_bp',
	-opt_handler_sub=>\&opt_handler_exec_rpmbuild_bp),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	-priority => PRIORITY_TRY_RPMBUILD_PREP_CLEANUP,
	-name => 'cleanup_rpmbuild_bp',
	    -opt_handler_sub=>\&opt_handler_cleanup_rpmbuild_bp),
	);
}

sub opt_handler_exec_rpmbuild_bp {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    unless ($opt->{-opt_try_rpmbuild_prep}) {
	return RPM::Source::TransformContainer::TRANSFORMATION_OK;
	print STDERR 'Transformation::TryRPMBuildPrep: rpmbuild_bp(): disabled ',".\n" if $verbose > 1;
    }
    #my $macros=$spec->macros();
    my $geardir=$spec->{GEARDIR};
    if ($geardir) {
	print STDERR "Fatal: TryRPMBuildPrep: in gear repository: not implemented yet\n";
	return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
    }
    my $sourcedir=$spec->_get_sourcedir();
    -d $sourcedir || die "FATAL: TryRPMBuildPrep: SOURCEDIR $sourcedir does not exist.\n";
    my $tmp_output_dir=File::Temp->newdir();
    $opt->{-try_rpmbuild_prep_tmpdir}=$tmp_output_dir;
    my $write_rpm_ok=$spec->write_rpm(
	'-rpmbuild_cmd' => 'rpmbuild',
	'-rpmbuild_buildopt' => $opt->{-opt_try_rpmbuild_stagearg},
	'-rpmbuild_extraopt_ref' => ['--define', '_builddir '.$tmp_output_dir]);
    unless ($write_rpm_ok) {
	print STDERR "Transformation::TryRPMBuildPrep: rpmbuild_bp() failed.\n" if $verbose;
	return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
    }
    my @unpack=glob("$tmp_output_dir/*");
    @unpack=grep {!m,/__MACOSX$,} @unpack if @unpack>1;
    if (1==@unpack and -d $unpack[0]) {
	$opt->{-try_rpmbuild_prep_sourcetree}=$unpack[0];
    } else {
	print STDERR 'info: TryRPMBuildPrep: rpmbuild_bp(): no explicit buildsubdir in ',$tmp_output_dir,'. Found: ',join(' ',@unpack),".\n" if $verbose;
    }
    print STDERR 'info: TryRPMBuildPrep: unpacked at ',$tmp_output_dir,"\n" if $verbose>1;
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub opt_handler_cleanup_rpmbuild_bp {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    return RPM::Source::TransformContainer::TRANSFORMATION_OK unless $opt->{-opt_try_rpmbuild_prep};
    # File::Temp use CLEANUP=>1 by default; this use rmtree
    $opt->{-try_rpmbuild_prep_tmpdir}=undef;
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;

__END__
