package RPM::Source::Transformation::InputHandler::NameList;

use strict;
use warnings;

use RPM::Source::Transform;
use RPM::Source::Transformation::Component::InputNameCommon;
use RPM::Source::Transformation::Component;
our @ISA=qw/RPM::Source::Transformation::Component/;

$RPM::Source::TransformContainer::INPUT_HANDLER=__PACKAGE__;

our $verbose=0;
our $opt_input_nvr;
our $opt_input_spec_patch;

sub print_longopt {
    print "  Input options:
    --input-nvr-string  NVR (name-version-release) as input
    --input-nvr-spec-patch NVR.diff input, also should be applied to spec file
";
}

our @LONGOPT=(
	"input-nvr-string"   => \$opt_input_nvr,
	"input-nvr-spec-patch"   => sub {$opt_input_nvr=1, $opt_input_spec_patch=1},
    );

sub process_args {}

sub verify_args {
    my ($class)=@_;
    if (not @ARGV) {
	$class->usage();
	exit 0;
    }
    return 1;
}

package RPM::Source::Transform;

use strict;
use warnings;
use File::Basename;
# TODO: use it or not?
use RPM::Header;

sub transform {
    my ($class)=@_;
    foreach my $in (@ARGV) {
	&transform_name($class,$in);
    }
}

sub transform_name {
    my ($class, $in)=@_;
    my $nvr;
    my ($rpmname, $rpmfile);
    if (-e $in and not -d $in and $in=~/\.src.rpm$/) {
	$rpmfile=$in;
	my $rhref;
	eval {
	    $rhref=new RPM::Header $in;
	};
	die "$in is not an rpm file!: $@" if $@;
	die "$in is not a source rpm!" if ! $rhref->is_source();
	$rpmname=$rhref->{NAME};
    } elsif ($opt_input_nvr) {
	$rpmname=basename($in=~s/-([^-]+-[^-]+)$//r);
	unless ($rpmname) {
	    die "Oops! not a valid NVR string: $in\n";
	} else {
	    $nvr=$1;
	    $nvr=~s/\.(patch|diff)$// if $opt_input_spec_patch;
	}
	
    } else {
	$rpmname=basename($in);
	$rpmname=~s/\.(patch|diff)$// if $opt_input_spec_patch;
    }
    
    my $converter_opt=RPM::Source::TransformContainer->mk_opt();
    push @{$converter_opt->{-opt_spec_apply_patch}}, $in if $opt_input_spec_patch;
    $converter_opt->{-input_handler_arg_input_name}=$rpmname;
    $converter_opt->{-input_handler_arg_input_nvr}=$nvr if $nvr;

    my $resourceobj=RPM::Source::TransformContainer->mk_resourceobj();
    my $spec=$RPM::Source::TransformContainer::Name2SpecProvider->name2spec($rpmname,$converter_opt,$resourceobj);
    RPM::Source::TransformContainer->transform_once($spec,undef,$converter_opt,$resourceobj);
}

1;

