package RPM::Source::Transformation::InputHandler::SRPMSpec;

use strict;
use warnings;

use RPM::Source::Transform;
use RPM::Source::Tools::File2RPMEditor;
use RPM::Source::Transformation::RPMEditorProvider;
use RPM::Source::Transformation::Component;
our @ISA=qw/RPM::Source::Transformation::Component/;
$RPM::Source::TransformContainer::INPUT_HANDLER=__PACKAGE__;

# TODO: make them $opt{}'ed and documented in help
our ($opt_sourcedir);

sub print_longopt {
    print "  SRPM input options:
	[--rpm] /path/to/src.rpm
	[--spec] /path/to/spec
	--sourcedir <dir> - default SOURCEDIR location for the provided spec file.
";
}

our @LONGOPT=(
    'rpm=s'	=> sub { push @RPM::Source::Transform::INSRC, [ SOURCERPM => $_[1]]},
    'sourcedir=s' => sub {$opt_sourcedir=$_[1]},
    'spec=s'	=> sub {push @RPM::Source::Transform::INSRC, defined $opt_sourcedir ? [SPECFILE => $_[1], SOURCEDIR=> $opt_sourcedir] : [ SPECFILE => $_[1]]}
    );

sub process_args {
    #my ($self)=@_;
    map {push @RPM::Source::Transform::INSRC, &RPM::Source::Tools::File2RPMEditor::file2rpmeditoropts($_, $opt_sourcedir)} @ARGV;
}

sub verify_args {
    #my ($self)=@_;
    unless (@RPM::Source::Transform::INSRC) {
	print STDERR "ERROR: please, specify [ --rpm | --spec ] /path/to/file argument.\n";
	return 0;
    }
    my $found=1;
    foreach my $input (@RPM::Source::Transform::INSRC) {
	if (! -e $input->[1]) {
	    $found=0;
	    warn "ERROR: ($input->[0]) $input->[1] not found\n";
	}
    }
    return $found;
}

package RPM::Source::Transform;

use strict;
use warnings;

our @INSRC;

sub transform {
    #my ($self)=@_;
    while (@INSRC) {
	my $spec=$RPM::Source::TransformContainer::RPMEditorProvider->rpmeditoropts2spec( shift @INSRC );
	my $parent=$RPM::Source::TransformContainer::RPMEditorProvider->rpmeditoropts2parent();
	my $opt_transform=RPM::Source::TransformContainer->mk_opt();
	my $resourceobj=RPM::Source::TransformContainer->mk_resourceobj();
	# debug opt
	# use Data::Dumper;
	# print Dumper($opt);
	RPM::Source::TransformContainer->transform_once($spec,$parent,$opt_transform,$resourceobj);
    };
}

1;
