# depends on Resource::Datapath, can't be an early resource
package RPM::Source::Transformation::Resource::FlagLoader;

use strict;
use warnings;
use Carp;
use RPM::Source::Tools::LoadFlag qw(read_list_as_flaghash read_list_as_arrayhash read_list_as_one2one);
use RPM::Source::Transformation::Resource;
use RPM::Source::Transformation::Resource::Datapath;
our @ISA=qw/RPM::Source::Transformation::Resource/;

our $verbose=0;

push @RPM::Source::TransformContainer::RESOURCE_CLASSES, __PACKAGE__;

use Exporter 'import';
our @EXPORT = qw(
	FLAG_RESOURCE_LIST_AS_FLAG
	FLAG_RESOURCE_LIST_AS_ARRAY
	FLAG_RESOURCE_LIST_AS_ONE2ONE
);
use constant {
	FLAG_RESOURCE_LIST_AS_FLAG	=> 17,
	FLAG_RESOURCE_LIST_AS_ARRAY	=> 18,
	FLAG_RESOURCE_LIST_AS_ONE2ONE	=> 19,
};

my %FLAGS;
my %TYPES;

our $load_flags=1;

sub print_longopt {
    print "	--no-flagdb do not load flags/* databases; default is load.
";
}

our @LONGOPT=(
    'flagdb!'	=> \$load_flags,
);


# static method
sub register_flagdb {
    my ($dbname,$type)=@_;
    die "Oops! register_flagdb: no type supplied for $dbname" unless $type;
    return unless $RPM::Source::Transformation::Resource::FlagLoader::load_flags;
    print STDERR "DEBUG: register_flagdb: registered $dbname as $type\n" if $verbose;
    $TYPES{$dbname}=$type;
}

package Source::Shared::ResourceContainer;

use strict;
use warnings;

# $dbname is mandatory;
# $name is optional; it was used in BundleImport transformations, but not now
sub get_flagdb {
    my ($resourceobj, $dbname,
	# do not use name!
	$name )=@_;
    return unless $RPM::Source::Transformation::Resource::FlagLoader::load_flags;
    my $hash=$resourceobj->__load_flagdb($dbname);
    unless ($hash) {
	print STDERR "DEBUG: get_flagdb: no db found for $dbname\n"
	    if $RPM::Source::Transformation::Resource::FlagLoader::verbose;
	return;
    }
    return $hash->{$name} if defined $name;
    foreach my $rname ($resourceobj->datapath_get_resource_names) {
	return $hash->{$rname} if exists $hash->{$rname};
    }
}

sub __load_flagdb {
    my ($resourceobj,$dbname)=@_;
    my $hash=$FLAGS{$dbname};
    if (!$hash) {
	my $type=$TYPES{$dbname};
	die "Oops! Resource::FlagLoader:load_flagdb: dbname $dbname is not registered" unless defined $type;
	$hash={};
	foreach my $dir (@{$resourceobj->flagsdirref()}) {
	    my @candidates=glob("$dir/$dbname");
	    print STDERR "DEBUG: __load_flagdb: dir=$dir candidates=",join(':',@candidates),"\n" if $RPM::Source::Transformation::Resource::FlagLoader::verbose>2;
	    next unless @candidates;
	    next if ! -e $candidates[0];
	    if ($type == RPM::Source::Transformation::Resource::FlagLoader::FLAG_RESOURCE_LIST_AS_FLAG) {
		&RPM::Source::Tools::LoadFlag::read_list_as_flaghash($hash,@candidates);
	    } elsif ($type == RPM::Source::Transformation::Resource::FlagLoader::FLAG_RESOURCE_LIST_AS_ARRAY) {
		&RPM::Source::Tools::LoadFlag::read_list_as_arrayhash($hash,@candidates);
	    } elsif ($type == RPM::Source::Transformation::Resource::FlagLoader::FLAG_RESOURCE_LIST_AS_ONE2ONE) {
		&RPM::Source::Tools::LoadFlag::read_list_as_one2one($hash,@candidates);
	    } else {
		Carp::croak "Oops! Resource::FlagLoader: $dbname: unknown type $type";
	    }
	    print STDERR "DEBUG: __load_flagdb: loaded: dir=$dir candidates=",join(':',@candidates),"\n" if $RPM::Source::Transformation::Resource::FlagLoader::verbose>2;
	    last;
	}
	$FLAGS{$dbname}=$hash;
    }
    return $hash;
}


1;
