package RPM::Source::Transformation::Writer::Basic;

# common component of writers

use strict;
use warnings;
use Cwd qw(getcwd);
use RPM::Source::Transformation::Component;
our @ISA=qw/RPM::Source::Transformation::Component/;
push @RPM::Source::TransformContainer::SUPPORT_CLASSES, __PACKAGE__;
$RPM::Source::TransformContainer::WRITER=__PACKAGE__;

our $verbose=1;

# opt'ified, seems to be not used as defaults
our (
     $opt_inplace, $opt_output_hasher_tar,
     # TODO: make them $opt{}'ed and documented in help
     $opt_write_spec,
);

our $opt_gear_git_add_spec;
our $opt_gear_commit_no_edit;

my $fallback_hasher_tar=1;
# TODO: make it $opt{}'ed and documented in help? but move, see below
my $opt_build_srpm=1;

# defaults
my $opt_output_srpm_dir;

sub print_longopt {
    print "  Basic Output options:
	-i|--inplace	Edit existing spec file (use together with --spec)
	-w|--write-spec <file>	Write modified spec file to <file>
	--hashertar	Write hasher-ready tar file instead of src.rpm file.
";
}

our @LONGOPT=(
    # TODO; hack; move option to transform container, to disable ANY writer
    # and rename to --dry-run
    'build-srpm!'	=> \$opt_build_srpm,

    'hashertar'	=> \$opt_output_hasher_tar,
    'w|writespec=s'	=> \$opt_write_spec,
    'i|inplace'	=> \$opt_inplace,
);

sub register_messages {
    my ($self,$extopt)=@_;
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-opt_output_spec_inplace => $opt_inplace,
	-output_srpm_dir => $opt_output_srpm_dir,
	-output_hasher_tar => $opt_output_hasher_tar,
    });
}

sub write {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $write_rpm_ok=1;
    print STDERR $spec->get_spec() if $verbose>4;
    my $macros=$spec->macros();
    my $srpm_output_dir=$opt->{-output_srpm_dir};
    # TODO: internal access!!! should be the write_rpm options!
    $spec->{OUTPUTDIR}=$macros->macro_subst($srpm_output_dir) if $srpm_output_dir;

    my $geardir=$spec->{GEARDIR};
    if ($opt_write_spec) {
	$spec->write_spec($macros->macro_subst($opt_write_spec));
    } elsif ($opt->{-opt_output_spec_inplace} or $geardir) {
	$spec->write_spec();
	if ($geardir) {
	    my $cwd=getcwd();
	    chdir $geardir;
	    &system_or_die('git', 'add', $spec->{SPECFILE}) if $opt_gear_git_add_spec;
	    &system_or_die('gear-commit','--no-edit') if $opt_gear_commit_no_edit;
	    chdir $cwd;
	}
    } else {
	my $write_hasher_tar=$opt->{-output_hasher_tar};
	if ($opt->{-safe_mode} and defined($write_hasher_tar) and not $write_hasher_tar) {
	    print STDERR "WARNING: writing hasher tar due to the safe mode.\n";
	    $write_hasher_tar=1;
	}
	my $sourcedir=$spec->_get_sourcedir();
	-d $sourcedir || die "FATAL: PkgWriter: SOURCEDIR $sourcedir does not exist.\nIf you are trying to change a spec file, use --inplace (-i) option.\n";

	if ($opt_build_srpm) {
	    if ($write_hasher_tar) {
		$spec->write_hasher_tar();
	    } else {
		$write_rpm_ok=$spec->write_rpm(-fallback_hasher_tar => $fallback_hasher_tar);
		#print STDERR "Transformation::PkgWriter: write_rpm() failed.\n" if $verbose and not $write_rpm_ok;
	    }
	}
    }
    return $write_rpm_ok;
}

sub system_or_die {
    my (@args)=@_;
    system(@args)==0 or die join(' ',@args)." failed";
}

1;
__END__

=pod

=head1	CLI SYNOPSIS

[B<--hashertar>]

=head1	CLI OPTIONS

=head2	CLI OPTIONS Output options

=over

=item	B<--hashertar>

Generate hasher-ready .tar file instead of src.rpm file.
Some src.rpm require you to install extra rpm-build-* or rpm-macros-* packages
or else rpmbuild -bs --nodeps will fail. On the other hand, generating hasher-ready .tar
file does not require installing rpm-build-* or rpm-macros-* packages, but
this tar file should be first processed in hasher.
Use --hashertar option to avoid polluting your working environment with
unnecessary rpm-build-* or rpm-macros-* packages.

=back

=cut

